@@ -115,6 +115,12 @@ class Base:
             self._read(_parse_tsv)
         )
         static([Type.File()], Type.Any(), "read_json")(self._read(_parse_json))
+        static([Type.File()], Type.Map((Type.String(), Type.String())), "read_object")(
+            self._read(_parse_object)
+        )
+        static([Type.File()], Type.Array(Type.Map((Type.String(), Type.String()))), "read_objects")(
+            self._read(_parse_objects)
+        )
 
         # polymorphically typed stdlib functions which require specialized
         # infer_type logic
@@ -315,7 +321,7 @@ def _parse_lines(s: str) -> Value.Array:
 
 
 def _parse_boolean(s: str) -> Value.Boolean:
-    s = s.rstrip()
+    s = s.rstrip().lower()
     if s == "true":
         return Value.Boolean(True)
     if s == "false":
@@ -324,17 +330,42 @@ def _parse_boolean(s: str) -> Value.Boolean:
 
 
 def _parse_tsv(s: str) -> Value.Array:
-    # TODO: should a blank line parse as [] or ['']?
     ans = [
         Value.Array(
             Type.Array(Type.String()), [Value.String(field) for field in line.value.split("\t")]
         )
         for line in _parse_lines(s).value
+        if line
     ]
     # pyre-ignore
     return Value.Array(Type.Array(Type.String()), ans)
 
 
+def _parse_objects(s: str) -> Value.Array:
+    strmat = _parse_tsv(s)
+    if len(strmat.value) < 1 or len(strmat.value[0].value) < 1:
+        return Value.Array(Type.Map((Type.String(), Type.String())), [])
+    keys = strmat.value[0].value
+    literal_keys = set(key.value for key in strmat.value[0].value if key.value)
+    if len(literal_keys) < len(keys):
+        raise Error.InputError("read_objects(): file has empty or duplicate column names")
+    maps = []
+    for row in strmat.value[1:]:
+        if len(row.value) != len(keys):
+            raise Error.InputError("read_objects(): file's tab-separated lines are ragged")
+        maps.append(Value.Map((Type.String(), Type.String()), list(zip(keys, row.value))))
+    return Value.Array(Type.Map((Type.String(), Type.String())), maps)
+
+
+def _parse_object(s: str) -> Value.Map:
+    maps = _parse_objects(s)
+    if len(maps.value) != 1:
+        raise Error.InputError("read_object(): file must have exactly one object")
+    map0 = maps.value[0]
+    assert isinstance(map0, Value.Map)
+    return map0
+
+
 def _parse_map(s: str) -> Value.Map:
     keys = set()
     ans = []
