@@ -14,6 +14,7 @@ import shutil
 import re
 from typing import Tuple, List, Dict, Optional, Callable, Iterable, Set, Any, Union
 from contextlib import ExitStack
+from docker.errors import BuildError as DockerBuildError
 
 from .. import Error, Type, Env, Value, StdLib, Tree, Expr, _util
 from .._util import (
@@ -102,10 +103,9 @@ def run_local_task(
 
         container = None
         try:
-            input_digest = cache.get_digest_for_inputs(inputs)
-            task_digest = cache.get_digest_for_task(task)
+            cache_key = f"{task.name}/{task.digest}/{Value.digest_env(inputs)}"
             cached = cache.get(
-                key=f"{task.name}_{task_digest}/{input_digest}",
+                key=cache_key,
                 output_types=task.effective_outputs,
                 inputs=inputs,
             )
@@ -207,7 +207,7 @@ def run_local_task(
                 )
                 logger.notice("done")  # pyre-fixme
                 if not run_id.startswith("download-"):
-                    cache.put(f"{task.name}_{task_digest}", input_digest, outputs)
+                    cache.put(cache_key, outputs)
                 return (run_dir, outputs)
         except Exception as exn:
             logger.debug(traceback.format_exc())
@@ -426,7 +426,14 @@ def _eval_task_runtime(
     logger.debug(_("runtime values", **dict((key, str(v)) for key, v in runtime_values.items())))
     ans = {}
 
-    if "docker" in runtime_values:
+    if "inlineDockerfile" in runtime_values:
+        # join Array[String]
+        dockerfile = runtime_values["inlineDockerfile"]
+        if not isinstance(dockerfile, Value.Array):
+            dockerfile = Value.Array(dockerfile.type, [dockerfile])
+        dockerfile = "\n".join(elt.coerce(Type.String()).value for elt in dockerfile.value)
+        ans["inlineDockerfile"] = dockerfile
+    elif "docker" in runtime_values:
         ans["docker"] = runtime_values["docker"].coerce(Type.String()).value
 
     host_limits = container.__class__.detect_resource_limits(cfg, logger)
@@ -536,7 +543,7 @@ def _try_task(
                     )
                 )
                 interruptions += 1
-            elif not isinstance(exn, Terminated) and retries < max_retries:
+            elif not isinstance(exn, (Terminated, DockerBuildError)) and retries < max_retries:
                 logger.error(
                     _(
                         "failed task will be retried",
