@@ -5,6 +5,7 @@ import random
 import os
 import shutil
 import json
+import time
 import docker
 from testfixtures import log_capture
 from .context import WDL
@@ -766,3 +767,91 @@ class MiscRegressionTests(RunnerTestCase):
         euid = os.geteuid()
         for fn in outp["files_out"]:
             assert os.stat(fn).st_uid == euid
+
+
+class TestInlineDockerfile(RunnerTestCase):
+    @log_capture()
+    def test1(self, capture):
+        wdl = """
+        version development
+        workflow w {
+            call t
+        }
+        task t {
+            input {
+                Array[String]+ apt_pkgs
+                Float timestamp
+            }
+            command <<<
+                set -euxo pipefail
+                apt list --installed | tr '/' $'\t' | sort > installed.txt
+                sort "~{write_lines(apt_pkgs)}" > expected.txt
+                join -j 1 -v 2 installed.txt expected.txt > missing.txt
+                if [ -s missing.txt ]; then
+                    >&2 cat missing.txt
+                    exit 1
+                fi
+            >>>
+            runtime {
+                inlineDockerfile: [
+                    "FROM ubuntu:20.04",
+                    "RUN apt-get -qq update && apt-get install -y ${sep(' ', apt_pkgs)}",
+                    "RUN touch ${timestamp}"
+                ]
+                maxRetries: 1
+            }
+        }
+        """
+        t = time.time()  # to ensure the image is built anew on every test run
+        self._run(wdl, {"t.apt_pkgs": ["samtools", "tabix"], "t.timestamp": t})
+        self._run(wdl, {"t.apt_pkgs": ["samtools", "tabix"], "t.timestamp": t})
+        logs = [str(record.msg) for record in capture.records if str(record.msg).startswith("docker build cached")]
+        self.assertEqual(len(logs), 1)
+        self._run(wdl, {"t.apt_pkgs": ["bogusfake123"], "t.timestamp": t}, expected_exception=docker.errors.BuildError)
+
+
+class TestAbbreviatedCallInput(RunnerTestCase):
+
+    def test_docker(self):
+        caller = R"""
+        version development
+        workflow caller {
+            input {
+                String message
+                String docker
+            }
+            call contrived as contrived1 {
+                input:
+                message = "~{message}1",
+                docker
+            }
+            call contrived as contrived2 {
+                input:
+                message = "~{message}2",
+                docker
+            }
+            output {
+                Array[String] results = [contrived1.result, contrived2.result]
+            }
+        }
+        task contrived {
+            input {
+                String message
+                String docker
+            }
+            command <<<
+                echo "~{message}"
+                cat /etc/issue
+            >>>
+            output {
+                String result = read_string(stdout())
+            }
+            runtime {
+                docker: docker
+            }
+        }
+        """
+        outputs = self._run(caller, {"message": "hello", "docker": "ubuntu:bionic"})
+        assert sum("18.04" in msg for msg in outputs["results"]) == 2
+        outputs = self._run(caller, {"message": "hello", "docker": "ubuntu:focal"})
+        assert sum("20.04" in msg for msg in outputs["results"]) == 2
